"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const graphql_1 = require("graphql");
const graphql_tag_1 = (0, tslib_1.__importDefault)(require("graphql-tag"));
const _1 = require(".");
const pubSchema = (0, graphql_1.buildSchema)(`
    type TestEvent {
      id: ID!
      name: String
    }
    type Query {
      TestEvent: TestEvent!
    }
  `);
test("valid event is published", () => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
    const publishCb = jest.fn();
    const publishErrorCb = jest.fn();
    const publishStartCb = jest.fn();
    const bus = new _1.VanillaEventBus({
        plugins: [
            {
                publishStartHook: (a) => {
                    publishStartCb(a.topic);
                    return {
                        publishErrorHook: () => publishErrorCb(a.topic),
                    };
                },
            },
        ],
        publisher: {
            schema: pubSchema,
            publish: (d) => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
                publishCb(d);
            }),
        },
    });
    yield bus.init();
    yield bus.publish({
        topic: "TestEvent",
        payload: {
            id: "123",
            name: "name",
        },
        metadata: {
            a: "b",
        },
    });
    expect(publishCb).toBeCalledTimes(1);
    expect(publishCb.mock.calls[0][0]).toMatchObject({
        baggage: expect.any(String),
        topic: "TestEvent",
    });
    const errorCb = jest.fn();
    yield bus
        .publish({
        topic: "NonExistingEvent",
        payload: {
            id: "123",
            name: "name",
        },
    })
        .catch((e) => {
        errorCb();
    });
    expect(errorCb).toBeCalled();
    expect(publishStartCb).toBeCalledTimes(2);
    expect(publishErrorCb).toBeCalledTimes(1);
}));
test("Allow invalid topic publishing", () => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
    const publishCb = jest.fn();
    const bus = new _1.VanillaEventBus({
        publisher: {
            schema: pubSchema,
            publish: (d) => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
                publishCb(d);
            }),
            allowInvalidTopic: true,
        },
    });
    yield bus.init();
    yield bus.publish({
        topic: "NonExistingTopic",
        payload: {},
    });
}));
test.only("valid events are consumed", () => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
    const consumeCb = jest.fn();
    let cbRef;
    const consumeErrCb = jest.fn();
    const consumeStartCb = jest.fn();
    const consumeEndCb = jest.fn();
    const bus = new _1.VanillaEventBus({
        plugins: [
            {
                consumeStartHook: (a) => {
                    consumeStartCb(a.topic);
                    return {
                        consumeErrorHook: (e) => consumeErrCb(a.topic, e.message),
                        consumeEndHook: () => {
                            consumeEndCb(a.topic);
                        },
                    };
                },
            },
        ],
        subscriber: {
            cb: consumeCb,
            getSubscription: (t, cb) => {
                cbRef = cb;
            },
            queries: (0, graphql_tag_1.default) `
        query TestEvent {
          TestEvent {
            id
            name
          }
        }
      `,
            schema: pubSchema,
        },
    });
    yield bus.init();
    yield cbRef({
        payload: {
            id: "id",
            name: "name",
        },
        metadata: {},
    });
    expect(consumeCb).toBeCalledTimes(1);
    expect(consumeCb.mock.calls[0][0]).toMatchObject({
        metadata: {},
        payload: {
            id: "id",
            name: "name",
        },
        topic: "TestEvent",
    });
    expect(consumeStartCb).toBeCalledTimes(1);
    expect(consumeErrCb).toBeCalledTimes(0);
    expect(consumeEndCb).toBeCalledTimes(1);
    consumeErrCb.mockClear();
    consumeEndCb.mockClear();
    consumeStartCb.mockClear();
    try {
        // error in the payload
        yield cbRef({
            payload: {
                name: "name",
            },
            metadata: {},
        });
    }
    catch (e) { }
    expect(consumeStartCb).toBeCalledTimes(1);
    expect(consumeEndCb).toBeCalledTimes(1);
    expect(consumeErrCb).toBeCalledTimes(1);
}));
