"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const eventbus_utils_1 = require("./eventbus-utils");
const fs_1 = (0, tslib_1.__importDefault)(require("fs"));
const graphql_1 = require("graphql");
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const graphql_tag_1 = (0, tslib_1.__importDefault)(require("graphql-tag"));
const typeDef = fs_1.default.readFileSync(path_1.default.join(__dirname, "../data/events.graphql"), "utf-8");
const publisherSchema = (0, graphql_1.buildSchema)(typeDef);
describe("getTopicsFromDocument", () => {
    test("works", () => {
        const result = (0, eventbus_utils_1.getTopicsFromDocument)((0, graphql_tag_1.default) `
      query C {
        C {
          id
        }
      }
      query B {
        some {
          ...Cool
        }
      }
      fragment Cool on Cool {
        name
      }
    `);
        expect(Object.keys(result)).toMatchInlineSnapshot(`
      Array [
        "C",
        "some",
      ]
    `);
    });
    test("Disallow mutation", () => {
        expect(() => (0, eventbus_utils_1.getTopicsFromDocument)((0, graphql_tag_1.default) `
        mutation whatver {
          cool
        }
      `)).toThrowErrorMatchingInlineSnapshot(`
"Event query must be of query type: mutation whatver {
  cool
}"
`);
    });
    test("Single query within a query", () => {
        expect(() => (0, eventbus_utils_1.getTopicsFromDocument)((0, graphql_tag_1.default) `
        query whatver {
          cool
          cool
        }
      `)).toThrow();
    });
});
describe("Query validation", () => {
    test("works 2", () => {
        expect(() => (0, eventbus_utils_1.validateQueries)(publisherSchema, [
            (0, graphql_tag_1.default) `
          query Complex {
            Complex {
              id
            }
          }

          query ClosedGroupJoinRequestResponseEvent {
            ClosedGroupJoinRequestResponseEvent {
              id
            }
          }
        `,
        ])).not.toThrow();
    });
    test("works", () => {
        expect(() => (0, eventbus_utils_1.validateQueries)(publisherSchema, [
            (0, graphql_tag_1.default) `
          query complex {
            A: SignUpEvent {
              id
            }
            B: SignUpEvent {
              id
            }
          }
        `,
        ])).not.toThrow();
    });
    test("Invalid event query throws", () => {
        expect(() => (0, eventbus_utils_1.validateQueries)(publisherSchema, [
            (0, graphql_tag_1.default) `
          query bad {
            bad {
              name
            }
          }
        `,
        ])).toThrow();
    });
    test("Deprecated field query logs warning", () => {
        expect(() => (0, eventbus_utils_1.validateQueries)(publisherSchema, [
            (0, graphql_tag_1.default) `
          query complex {
            Complex {
              groupId
            }
          }
        `,
        ])).not.toThrow();
    });
});
describe("getRootQueryNames", () => {
    test("works", () => {
        const schema = (0, graphql_1.buildSchema)(typeDef);
        const topicNames = (0, eventbus_utils_1.getRootQueryFields)(schema);
        expect(topicNames).toMatchInlineSnapshot(`
      Array [
        "Complex",
        "B",
        "C",
        "ClosedGroupJoinRequestResponseEvent",
        "SignUpEvent",
        "EntityFlagEvent",
      ]
    `);
    });
    // test("Each root return type has and id and timestamp", () => {
    //   const badSchema =
    // })
    test("Nullable return type throws", () => {
        const badSchema = (0, graphql_1.buildSchema)(`
      type Query {
        A: String!
        B: String
      }
    `);
        expect(() => (0, eventbus_utils_1.getRootQueryFields)(badSchema)).toThrowErrorMatchingInlineSnapshot(`"You must specify all queries with non null response type"`);
    });
});
